@@ -8,6 +8,7 @@ mod context;
 mod dijkstra;
 mod files;
 mod graph;
+mod guess_language;
 mod hunks;
 mod inline;
 mod line_parser;
@@ -15,6 +16,7 @@ mod lines;
 mod positions;
 mod side_by_side;
 mod style;
+mod summary;
 mod syntax;
 mod tree_sitter_parser;
 
@@ -22,6 +24,7 @@ mod tree_sitter_parser;
 extern crate log;
 
 use crate::hunks::{matched_pos_to_hunks, merge_adjacent};
+use guess_language::guess;
 use log::info;
 use mimalloc::MiMalloc;
 
@@ -34,7 +37,8 @@ static GLOBAL: MiMalloc = MiMalloc;
 
 use atty::Stream;
 use clap::{crate_version, App, AppSettings, Arg};
-use std::{env, ffi::OsStr, path::Path};
+use std::{env, path::Path};
+use summary::DiffResult;
 use typed_arena::Arena;
 use walkdir::WalkDir;
 
@@ -182,34 +186,35 @@ fn main() {
 
     match parse_args() {
         Mode::DumpTreeSitter { path } => {
-            let extension = Path::new(&path).extension();
-            let extension = extension.unwrap_or_else(|| OsStr::new(""));
-            match tsp::from_extension(extension) {
-                Some(ts_lang) => {
-                    let bytes = read_or_die(&path);
-                    let src = String::from_utf8_lossy(&bytes).to_string();
+            let path = Path::new(&path);
+            let bytes = read_or_die(path);
+            let src = String::from_utf8_lossy(&bytes).to_string();
+            match guess(path, &src) {
+                Some(lang) => {
+                    let ts_lang = tsp::from_language(lang);
                     let (tree, _) = tsp::parse_to_tree(&src, &ts_lang);
-                    tsp::print_tree(&tree);
+                    tsp::print_tree(&src, &tree);
                 }
                 None => {
-                    println!("No tree-sitter parser for extension: {:?}", extension);
+                    eprintln!("No tree-sitter parser for file: {:?}", path);
                 }
             }
         }
         Mode::DumpSyntax { path } => {
-            let extension = Path::new(&path).extension();
-            let extension = extension.unwrap_or_else(|| OsStr::new(""));
-            match tsp::from_extension(extension) {
-                Some(ts_lang) => {
-                    let bytes = read_or_die(&path);
-                    let src = String::from_utf8_lossy(&bytes).to_string();
+            let path = Path::new(&path);
+            let bytes = read_or_die(path);
+            let src = String::from_utf8_lossy(&bytes).to_string();
+
+            match guess(path, &src) {
+                Some(lang) => {
+                    let ts_lang = tsp::from_language(lang);
                     let arena = Arena::new();
                     let ast = tsp::parse(&arena, &src, &ts_lang);
                     init_info(&ast, &[]);
                     println!("{:#?}", ast);
                 }
                 None => {
-                    println!("No tree-sitter parser for extension: {:?}", extension);
+                    eprintln!("No tree-sitter parser for file: {:?}", path);
                 }
             }
         }
@@ -218,26 +223,38 @@ fn main() {
             lhs_path,
             rhs_path,
         } => {
-            if Path::new(&lhs_path).is_dir() && Path::new(&rhs_path).is_dir() {
-                diff_directories(&lhs_path, &rhs_path);
+            let lhs_path = Path::new(&lhs_path);
+            let rhs_path = Path::new(&rhs_path);
+
+            if lhs_path.is_dir() && rhs_path.is_dir() {
+                for diff_result in diff_directories(lhs_path, rhs_path) {
+                    print_diff_result(&diff_result);
+                }
             } else {
-                diff_file(&display_path, &lhs_path, &rhs_path);
+                let diff_result = diff_file(&display_path, lhs_path, rhs_path);
+                print_diff_result(&diff_result);
             }
         }
     };
 }
 
 /// Print a diff between two files.
-// TODO: prefer PathBuf to &str for paths.
-fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
+fn diff_file(display_path: &str, lhs_path: &Path, rhs_path: &Path) -> DiffResult {
     let lhs_bytes = read_or_die(lhs_path);
     let rhs_bytes = read_or_die(rhs_path);
 
     let lhs_binary = is_probably_binary(&lhs_bytes);
     let rhs_binary = is_probably_binary(&rhs_bytes);
     if lhs_binary || rhs_binary {
-        print!("{}", style::header(display_path, 1, 1, "binary"));
-        return;
+        return DiffResult {
+            path: display_path.into(),
+            language: None,
+            binary: true,
+            lhs_src: "".into(),
+            rhs_src: "".into(),
+            lhs_positions: vec![],
+            rhs_positions: vec![],
+        };
     }
 
     // TODO: don't replace tab characters inside string literals.
@@ -248,19 +265,27 @@ fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
         .to_string()
         .replace("\t", "    ");
 
-    let extension = Path::new(&display_path).extension();
-    let extension = extension.unwrap_or_else(|| OsStr::new(""));
-    let ts_lang = tsp::from_extension(extension);
+    // TODO: take a Path directly instead.
+    let path = Path::new(&display_path);
+
+    // Take the longer of the two files when guessing language. This
+    // is useful when we've added or removed a whole file.
+    let guess_src = if lhs_src.len() > rhs_src.len() {
+        &lhs_src
+    } else {
+        &rhs_src
+    };
+    let ts_lang = guess(path, guess_src).map(tsp::from_language);
 
     let arena = Arena::new();
     let (lang_name, lhs, rhs) = match ts_lang {
         Some(ts_lang) => (
-            ts_lang.name,
+            Some(ts_lang.name.into()),
             tsp::parse(&arena, &lhs_src, &ts_lang),
             tsp::parse(&arena, &rhs_src, &ts_lang),
         ),
         None => (
-            "text",
+            None,
             line_parser::parse(&arena, &lhs_src),
             line_parser::parse(&arena, &rhs_src),
         ),
@@ -272,45 +297,14 @@ fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
     let lhs_positions = change_positions(&lhs_src, &rhs_src, &lhs);
     let rhs_positions = change_positions(&rhs_src, &lhs_src, &rhs);
 
-    let hunks = matched_pos_to_hunks(&lhs_positions, &rhs_positions);
-    let hunks = merge_adjacent(
-        &hunks,
-        &lhs_positions,
-        &rhs_positions,
-        lhs_src.max_line(),
-        rhs_src.max_line(),
-    );
-
-    if hunks.is_empty() {
-        println!("{}", style::header(display_path, 1, 1, lang_name));
-        if lang_name == "text" {
-            println!("No changes.\n");
-        } else {
-            println!("No syntactic changes.\n");
-        }
-        return;
-    }
-
-    if env::var("INLINE").is_ok() {
-        println!("{}", style::header(display_path, 1, 1, lang_name));
-
-        println!(
-            "{}",
-            inline::display(&lhs_src, &rhs_src, &lhs_positions, &rhs_positions, &hunks)
-        );
-    } else {
-        println!(
-            "{}",
-            side_by_side::display_hunks(
-                &hunks,
-                display_path,
-                lang_name,
-                &lhs_src,
-                &rhs_src,
-                &lhs_positions,
-                &rhs_positions,
-            )
-        );
+    DiffResult {
+        path: display_path.into(),
+        language: lang_name,
+        binary: false,
+        lhs_src,
+        rhs_src,
+        lhs_positions,
+        rhs_positions,
     }
 }
 
@@ -318,8 +312,9 @@ fn diff_file(display_path: &str, lhs_path: &str, rhs_path: &str) {
 /// pairwise.
 ///
 /// When more than one file is modified, the hg extdiff extension passes directory
-/// paths with the all the modified files.  fn
-fn diff_directories(lhs_dir: &str, rhs_dir: &str) {
+/// paths with the all the modified files.
+fn diff_directories(lhs_dir: &Path, rhs_dir: &Path) -> Vec<DiffResult> {
+    let mut res = vec![];
     for entry in WalkDir::new(lhs_dir).into_iter().filter_map(Result::ok) {
         let lhs_path = entry.path();
         if lhs_path.is_dir() {
@@ -331,10 +326,62 @@ fn diff_directories(lhs_dir: &str, rhs_dir: &str) {
         let rel_path = lhs_path.strip_prefix(lhs_dir).unwrap();
         let rhs_path = Path::new(rhs_dir).join(rel_path);
 
-        diff_file(
-            &rel_path.to_string_lossy(),
-            &lhs_path.to_string_lossy(),
-            &rhs_path.to_string_lossy(),
+        res.push(diff_file(&rel_path.to_string_lossy(), &lhs_path, &rhs_path));
+    }
+    res
+}
+
+fn print_diff_result(summary: &DiffResult) {
+    if summary.binary {
+        print!("{}", style::header(&summary.path, 1, 1, "binary"));
+        return;
+    }
+
+    let hunks = matched_pos_to_hunks(&summary.lhs_positions, &summary.rhs_positions);
+    let hunks = merge_adjacent(
+        &hunks,
+        &summary.lhs_positions,
+        &summary.rhs_positions,
+        summary.lhs_src.max_line(),
+        summary.rhs_src.max_line(),
+    );
+
+    let lang_name = summary.language.clone().unwrap_or_else(|| "text".into());
+    if hunks.is_empty() {
+        println!("{}", style::header(&summary.path, 1, 1, &lang_name));
+        if lang_name == "text" {
+            println!("No changes.\n");
+        } else {
+            println!("No syntactic changes.\n");
+        }
+        return;
+    }
+
+    if env::var("INLINE").is_ok() {
+        println!("{}", style::header(&summary.path, 1, 1, &lang_name));
+
+        println!(
+            "{}",
+            inline::display(
+                &summary.lhs_src,
+                &summary.rhs_src,
+                &summary.lhs_positions,
+                &summary.rhs_positions,
+                &hunks
+            )
+        );
+    } else {
+        println!(
+            "{}",
+            side_by_side::display_hunks(
+                &hunks,
+                &summary.path,
+                &lang_name,
+                &summary.lhs_src,
+                &summary.rhs_src,
+                &summary.lhs_positions,
+                &summary.rhs_positions,
+            )
         );
     }
 }
