@@ -4,6 +4,7 @@ use colored::{Color, Colorize};
 use std::{
     cmp::max,
     collections::{HashMap, HashSet},
+    env,
 };
 
 use crate::{
@@ -17,8 +18,17 @@ use crate::{
 
 const SPACER: &str = " ";
 
-fn term_width() -> Option<usize> {
-    term_size::dimensions().map(|(w, _)| w)
+/// Choose the display width: honour environment variables, then try
+/// to autodetect, or fall back to a sensible default.
+fn display_width() -> usize {
+    // TODO: document in manual.
+    if let Ok(s) = env::var("DFT_WIDTH") {
+        if let Ok(i) = s.parse::<usize>() {
+            return i;
+        }
+    }
+
+    term_size::dimensions().map(|(w, _)| w).unwrap_or(80)
 }
 
 /// Split `s` by newlines, but guarantees that the output is nonempty.
@@ -238,7 +248,7 @@ pub fn display_hunks(
         let no_rhs_changes = hunk.lines.iter().all(|(_, r)| r.is_none());
         let same_lines = aligned_lines.iter().all(|(l, r)| l == r);
 
-        let widths = Widths::new(term_width().unwrap_or(80), &aligned_lines, lhs_src, rhs_src);
+        let widths = Widths::new(display_width(), &aligned_lines, lhs_src, rhs_src);
         for (lhs_line_num, rhs_line_num) in aligned_lines {
             let (display_lhs_line_num, display_rhs_line_num) = display_line_nums(
                 lhs_line_num,
@@ -251,26 +261,41 @@ pub fn display_hunks(
             );
 
             if no_lhs_changes {
-                let rhs_line = &rhs_colored_lines[rhs_line_num.expect("Should have RHS line").0];
-                if same_lines {
-                    // Don't print the line numbers in two columns if
-                    // they're all the same.
-                    out_lines.push(format!("{}{}", display_rhs_line_num, rhs_line));
-                } else {
-                    out_lines.push(format!(
-                        "{}{}{}",
-                        display_lhs_line_num, display_rhs_line_num, rhs_line
-                    ));
+                match rhs_line_num {
+                    Some(rhs_line_num) => {
+                        let rhs_line = &rhs_colored_lines[rhs_line_num.0];
+                        if same_lines {
+                            out_lines.push(format!("{}{}", display_rhs_line_num, rhs_line));
+                        } else {
+                            out_lines.push(format!(
+                                "{}{}{}",
+                                display_lhs_line_num, display_rhs_line_num, rhs_line
+                            ));
+                        }
+                    }
+                    None => {
+                        // We didn't have any changed RHS lines in the
+                        // hunk, but we had some contextual lines that
+                        // only occurred on the LHS (e.g. extra newlines).
+                        out_lines.push(format!("{}{}", display_rhs_line_num, display_rhs_line_num));
+                    }
                 }
             } else if no_rhs_changes {
-                let lhs_line = &lhs_colored_lines[lhs_line_num.expect("Should have LHS line").0];
-                if same_lines {
-                    out_lines.push(format!("{}{}", display_lhs_line_num, lhs_line));
-                } else {
-                    out_lines.push(format!(
-                        "{}{}{}",
-                        display_lhs_line_num, display_rhs_line_num, lhs_line
-                    ));
+                match lhs_line_num {
+                    Some(lhs_line_num) => {
+                        let lhs_line = &lhs_colored_lines[lhs_line_num.0];
+                        if same_lines {
+                            out_lines.push(format!("{}{}", display_lhs_line_num, lhs_line));
+                        } else {
+                            out_lines.push(format!(
+                                "{}{}{}",
+                                display_lhs_line_num, display_rhs_line_num, lhs_line
+                            ));
+                        }
+                    }
+                    None => {
+                        out_lines.push(format!("{}{}", display_lhs_line_num, display_rhs_line_num));
+                    }
                 }
             } else {
                 let lhs_line = match lhs_line_num {
